"""
###################################################
Program One Channel 
 
version: 2.1
 
A program to construct a simple channel/ pipe as a rectangular block.
The program creates the geometry and mesh according to
a model for a CFD application in OpenFOAM.

 - tested with SALOME 7.4.0
 - defines the required surface and volume groups
 - creates the mesh
 - gradient at channel walls (channel consist of 4 cubes)
 
 new in version 2.1
 - width and height can have different mesh resolution (for flat shapes)
 
 new in version 2.0
 - inlet and outlet region are optional (see input parameter 6)
 - mesh: number of segments for lenght of channel is now flexible (see input parameter 12)

Uwe Reimer, Forschungszentrum Juelich / Germany
14 June 2016 
###################################################
"""
import salome
salome.salome_init()
import GEOM
from salome.geom import geomBuilder
geompy = geomBuilder.New(salome.myStudy)
import SMESH
from salome.smesh import smeshBuilder
smesh = smeshBuilder.New(salome.myStudy)
#############################################################################################
### INPUT PARAMETERS from users
#############################################################################################
myinput = []	# container for input variables
#######################################
## GEOMETRY
#######################################
myinput.append(0.0)	# origin coordinate x	/ 0
myinput.append(0.0) # origin coordinate y	/ 1
myinput.append(0.0) # origin coordinate z	/ 2

myinput.append(5.0e-3)		# channel width x in m		/ 3
myinput.append(85.0e-3)		# channel length y in m		/ 4
myinput.append(1.0e-3)		# channel height z in m		/ 5

myinput.append(True)		# (True) or (False) to have additional inlet and outlet / 6
myinput.append(5.0e-3)		# additional length of inlet  in m	/ 7
myinput.append(5.0e-3)		# additional length of outlet  in m	/ 8

#######################################
## MESH
#######################################
myinput.append(16) 			# mesh: (half of) number of segments per channel width	/ 9
myinput.append(True)		#(True) or (False) mesh: use mesh gradient in channel	/ 10
myinput.append(20)			# mesh: gradient value 			/11		( factor for refinement (small numbers like 4 will give many cells with little gradient / large numbers like 40 will give nice gradient) )
myinput.append(4.0) 		# mesh: define number of segments for length of channel as multiplier of the size of segments per channel width  / 12
myinput.append(3) 			# mesh: (half of) number of segments per channel height	/ 13
#############################################################################################
### END INPUT PARAMETERS from users
#############################################################################################

#############################################################################################
### Names and lists for GROUP definition
#############################################################################################
## create an empty list for each group
# volume groups
fluid = []
myfluidnames = ['fluid']
# face groups
fwall = []
finlet = []
foutlet = []
fmembrane = []
myfacegroups = [fwall, finlet, foutlet, fmembrane]
myfacenames = ['wall', 'inlet', 'outlet', 'membrane']
# 
#############################################################################################
### internal parameters
#############################################################################################
myboxes = []	# container for all boxes
myboxpts = []	# container for all box points
allgroups = []  # container for all generated groups

#############################################################################################
### FUNCTION to define one box by 8 corners and assign a name
#############################################################################################
def DefBlock(mx, my, mz, x0, y0, z0, mybox):
	#: compute 8 points (8 corners) and add them to a list -> mybox (mybox1 must be an empty list !!) 
	# x-vector = mybox[0], mybox[1]
	# y-vector = mybox[0], mybox[3]
	# z-vector = mybox[0], mybox[6]
	global geompy
	
	pt = geompy.MakeVertex ( x0, y0, z0 )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( (x0 + mx), y0, z0 )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( (x0 + mx), (y0 + my), z0 )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( x0, (y0 + my), z0 )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( (x0 + mx), (y0 + my), (z0 + mz) )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( x0,(y0 + my), (z0 + mz) )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( x0, y0, (z0 + mz) )
	mybox.append(pt)
	
	pt = geompy.MakeVertex ( (x0 + mx), y0, (z0 + mz) )
	mybox.append(pt)

#############################################################################################
### FUNCTION to make geometry   -> with inlet / outlet region
#############################################################################################
def CreateCell(myboxes, myboxpts, myinput):
	#: Create GEOMETRY / everything is a grid of boxes
	global geompy
	
	xoff = 0.0		# offset in x (for construction)
	yoff = 0.0
	zoff = 0.0		# offset in z
	row = 0
	col = 0

	x0 = myinput[0]
	y0 = myinput[1]
	z0 = myinput[2]	
	x = myinput[3] / 2.0	# because channel is 4 boxes
	y = myinput[4]		# length
	z = myinput[5] / 2.0	# because channel is 4 boxes

	mylist = [ myinput[7], myinput[4], myinput[8] ]
	
	for m in mylist:
		y = m
	
		# box 1
		xoff = 0.0
		zoff = 0.0
		mybox = []					# container for set of 8 points (x,y,z)
		x0 = myinput[0] + xoff
		z0 = myinput[2]	+ zoff
		DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
		box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
		#geompy.addToStudy(box, 'box0')						# for visualization: add box to the geometry
		myboxes.append(box)						# add boxes to list
		myboxpts.append(mybox)					# add all points to list
		
		# box 2
		xoff = x
		zoff = 0.0
		mybox = []					# container for set of 8 points (x,y,z)
		x0 = myinput[0] + xoff
		z0 = myinput[2]	+ zoff
		DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
		box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
		#geompy.addToStudy(box, 'box1')						# for visualization: add box to the geometry
		myboxes.append(box)						# add boxes to list
		myboxpts.append(mybox)					# add all points to list
		
		# box 3
		xoff = x
		zoff = z
		mybox = []					# container for set of 8 points (x,y,z)
		x0 = myinput[0] + xoff
		z0 = myinput[2]	+ zoff
		DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
		box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
		#geompy.addToStudy(box, 'box2')						# for visualization: add box to the geometry
		myboxes.append(box)						# add boxes to list
		myboxpts.append(mybox)					# add all points to list

		# box 4
		xoff = 0.0
		zoff = z
		mybox = []					# container for set of 8 points (x,y,z)
		x0 = myinput[0] + xoff
		z0 = myinput[2]	+ zoff
		DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
		box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
		#geompy.addToStudy(box, 'box3')						# for visualization: add box to the geometry
		myboxes.append(box)						# add boxes to list
		myboxpts.append(mybox)					# add all points to list
	
		y0 = y0 + y
	
#############################################################################################
### FUNCTION to make geometry -> without inlet / outlet region
#############################################################################################
def CreateCell2(myboxes, myboxpts, myinput):
	#: Create GEOMETRY / everything is a grid of boxes
	global geompy
	
	xoff = 0.0		# offset in x (for construction)
	yoff = 0.0
	zoff = 0.0		# offset in z
	row = 0
	col = 0

	x0 = myinput[0]
	y0 = myinput[1]
	z0 = myinput[2]	
	x = myinput[3] / 2.0	# because channel is 4 boxes
	y = myinput[4]			# length
	z = myinput[5] / 2.0	# because channel is 4 boxes
	
	y = myinput[4]

	# box 1
	xoff = 0.0
	zoff = 0.0
	mybox = []					# container for set of 8 points (x,y,z)
	x0 = myinput[0] + xoff
	z0 = myinput[2]	+ zoff
	DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
	box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
	#geompy.addToStudy(box, 'box0')						# for visualization: add box to the geometry
	myboxes.append(box)						# add boxes to list
	myboxpts.append(mybox)					# add all points to list
	
	# box 2
	xoff = x
	zoff = 0.0
	mybox = []					# container for set of 8 points (x,y,z)
	x0 = myinput[0] + xoff
	z0 = myinput[2]	+ zoff
	DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
	box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
	#geompy.addToStudy(box, 'box1')						# for visualization: add box to the geometry
	myboxes.append(box)						# add boxes to list
	myboxpts.append(mybox)					# add all points to list
	
	# box 3
	xoff = x
	zoff = z
	mybox = []					# container for set of 8 points (x,y,z)
	x0 = myinput[0] + xoff
	z0 = myinput[2]	+ zoff
	DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
	box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
	#geompy.addToStudy(box, 'box2')						# for visualization: add box to the geometry
	myboxes.append(box)						# add boxes to list
	myboxpts.append(mybox)					# add all points to list

	# box 4
	xoff = 0.0
	zoff = z
	mybox = []					# container for set of 8 points (x,y,z)
	x0 = myinput[0] + xoff
	z0 = myinput[2]	+ zoff
	DefBlock(x, y, z, x0, y0, z0, mybox)		# create points
	box = geompy.MakeBoxTwoPnt(mybox[0], mybox[4])		# create the box by 2 points
	#geompy.addToStudy(box, 'box3')						# for visualization: add box to the geometry
	myboxes.append(box)						# add boxes to list
	myboxpts.append(mybox)					# add all points to list

	y0 = y0 + y

#############################################################################################
### FUNCTION to make groups for faces (inlet, outlet) and volume parts	-> with inlet / outlet region
#############################################################################################
def CreateGroups(myboxpts, myinput, mychannel, fluid, myfluidnames, myfacegroups, myfacenames, allgroups):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	global geompy
	
	"""
	# volume groups
	fluid 		-> 0	
	# face groups
	fwall			-> 0
	finlet	 		-> 1
	foutlet  		-> 2
	fmembrane	 	-> 3
	"""	
	
	##################################################################################################	
	# Sub-shapes will be sorted by coordinates of their gravity centers
	# this list of subshapes is sorted:  increasing z / increasing x / increasing y 
	mylist = geompy.SubShapeAllSortedCentres(mychannel, geompy.ShapeType["SOLID"]) 
		
	## create solid (fluid) group from list
	group = geompy.CreateGroup(mychannel, geompy.ShapeType["SOLID"])
	for a in mylist:
		id = geompy.GetSubShapeID(mychannel, a)
		geompy.AddObject(group, id)
	
	geompy.addToStudyInFather(mychannel, group, myfluidnames[0])
	allgroups.append(group)
	
	# here we assign face groups
	# we have to re-sort it to fit the correct order created in function CreateCell
	newboxes = []		 	# list to collect subshapes in the correct order (according to function CreateCell )
	newboxes.append(mylist[0])
	newboxes.append(mylist[2])
	newboxes.append(mylist[3])
	newboxes.append(mylist[1])
	newboxes.append(mylist[4])
	newboxes.append(mylist[6])
	newboxes.append(mylist[7])
	newboxes.append(mylist[5])
	newboxes.append(mylist[8])
	newboxes.append(mylist[10])
	newboxes.append(mylist[11])
	newboxes.append(mylist[9])
	
	## start running through the geometry	
	 
	i = 0
	while ( i < len(mylist) ):
		m = myboxpts[i]
		# these are the faces of box with number i
		front = geompy.GetFaceByPoints(newboxes[i], m[0], m[1], m[6], m[7] )
		back = geompy.GetFaceByPoints(newboxes[i], m[2], m[3], m[4], m[5] )
		bottom = geompy.GetFaceByPoints(newboxes[i], m[0], m[1], m[2], m[3] )
		top = geompy.GetFaceByPoints(newboxes[i], m[4], m[5], m[6], m[7] )
		left = geompy.GetFaceByPoints(newboxes[i], m[0], m[3], m[5], m[6] )
		right = geompy.GetFaceByPoints(newboxes[i], m[1], m[2], m[4], m[7] )
		
		# inlet
		if ( i < 4 ):
			myfacegroups[1].append(front)
			
		# outlet
		if ( i > 7 ):
			myfacegroups[2].append(back)
		
		# pre-inlet and post-outlet walls
		if ( i == 0 or i == 8 or i == 4):
			myfacegroups[0].append(left)
			myfacegroups[0].append(bottom)
			
		if ( i == 1 or i == 9 or i == 5 ):
			myfacegroups[0].append(right)
			myfacegroups[0].append(bottom)
			
		if ( i == 2 or i == 10 ):
			myfacegroups[0].append(right)
			myfacegroups[0].append(top)
			
		if ( i == 3 or i == 11 ):
			myfacegroups[0].append(left)
			myfacegroups[0].append(top)
			
		# membrane / channel 
		if ( i == 7 ):
			myfacegroups[0].append(left)
			myfacegroups[3].append(top)
			
		if ( i == 6 ):
			myfacegroups[0].append(right)
			myfacegroups[3].append(top)
			
		i = i + 1
	
	## create face groups from lists
	x = 0
	for a in myfacegroups:	
		group = geompy.CreateGroup(mychannel, geompy.ShapeType["FACE"])
		mylist = a
		for i in mylist:
			id = geompy.GetSubShapeID(mychannel, i)
			geompy.AddObject(group, id)
	
		geompy.addToStudyInFather(mychannel, group, myfacenames[x])
		allgroups.append(group)
		x = x + 1

#############################################################################################
### FUNCTION to make groups for faces (inlet, outlet) and volume parts  -> without inlet / outlet region
#############################################################################################
def CreateGroups2(myboxpts, myinput, mychannel, fluid, myfluidnames, myfacegroups, myfacenames, allgroups):
	#: Create GROUPS to be addressed later as boundary for OpenFOAM
	global geompy
	
	"""
	# volume groups
	fluid 		-> 0	
	# face groups
	fwall			-> 0
	finlet	 		-> 1
	foutlet  		-> 2
	fmembrane	 	-> 3
	"""	
	
	##################################################################################################	
	# Sub-shapes will be sorted by coordinates of their gravity centers
	# this list of subshapes is sorted:  increasing z / increasing x / increasing y 
	mylist = geompy.SubShapeAllSortedCentres(mychannel, geompy.ShapeType["SOLID"]) 
		
	## create solid (fluid) group from list
	group = geompy.CreateGroup(mychannel, geompy.ShapeType["SOLID"])
	for a in mylist:
		id = geompy.GetSubShapeID(mychannel, a)
		geompy.AddObject(group, id)
	
	geompy.addToStudyInFather(mychannel, group, myfluidnames[0])
	allgroups.append(group)
	
	# here we assign face groups
	# we have to re-sort it to fit the correct order created in function CreateCell
	newboxes = []		 	# list to collect subshapes in the correct order (according to function CreateCell )
	newboxes.append(mylist[0])
	newboxes.append(mylist[2])
	newboxes.append(mylist[3])
	newboxes.append(mylist[1])
	
	
	## start running through the geometry	
	 
	i = 0
	while ( i < len(mylist) ):
		m = myboxpts[i]
		# these are the faces of box with number i
		front = geompy.GetFaceByPoints(newboxes[i], m[0], m[1], m[6], m[7] )
		back = geompy.GetFaceByPoints(newboxes[i], m[2], m[3], m[4], m[5] )
		bottom = geompy.GetFaceByPoints(newboxes[i], m[0], m[1], m[2], m[3] )
		top = geompy.GetFaceByPoints(newboxes[i], m[4], m[5], m[6], m[7] )
		left = geompy.GetFaceByPoints(newboxes[i], m[0], m[3], m[5], m[6] )
		right = geompy.GetFaceByPoints(newboxes[i], m[1], m[2], m[4], m[7] )
		
		# inlet
		myfacegroups[1].append(front)
			
		# outlet
		myfacegroups[2].append(back)
		
		if ( i == 0 ):
			myfacegroups[0].append(left)
			myfacegroups[0].append(bottom)
			
		if ( i == 1 ):
			myfacegroups[0].append(right)
			myfacegroups[0].append(bottom)
			
		if ( i == 2 ):
			myfacegroups[0].append(right)
			myfacegroups[3].append(top)
			
		if ( i == 3 ):
			myfacegroups[0].append(left)
			myfacegroups[3].append(top)
			
		i = i + 1
	
	## create face groups from lists
	x = 0
	for a in myfacegroups:	
		group = geompy.CreateGroup(mychannel, geompy.ShapeType["FACE"])
		mylist = a
		for i in mylist:
			id = geompy.GetSubShapeID(mychannel, i)
			geompy.AddObject(group, id)
	
		geompy.addToStudyInFather(mychannel, group, myfacenames[x])
		allgroups.append(group)
		x = x + 1

#############################################################################################
### FUNCTION to make MESH	-> with inlet / outlet region
#############################################################################################
def CreateMesh(myboxes, myboxpts, myinput, fuelcell, allgroups):
	#: Create the mesh
	#: 1) we initialize everything with 1 segment
	#: 2) define number of segments for length of channel
	#: 3) define number of segments for width of channel
	#: 4) define number of segments for height of channel, interconnect, GDL, electrolyte
	global smesh
	
	### first we initialize everything with 1 segment
	hexa = smesh.Mesh(fuelcell)
	hexa_1d = hexa.Segment()
	hexa_1d.NumberOfSegments(1) 
	
	### define number of segments for length of channel as multiplier of the size of segments per channel width  
	
	# inlet
	#myseg = int( myinput[7] / ( myinput[3] * myinput[12] ) ) * myinput[9] * 2   # based on width
	myseg = int( myinput[7] / ( myinput[5] * myinput[12] ) ) * myinput[13] * 2    # based on heigth
	if myseg == 0:
		myseg = 1
	i = 0 # box number 
	mybox = myboxpts[i]
	edge = geompy.GetEdge(fuelcell, mybox[0], mybox[3])		# length-y from box
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	# outlet
	i = 8 # box number 
	mybox = myboxpts[i]
	edge = geompy.GetEdge(fuelcell, mybox[0], mybox[3])		# length-y from box
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	# channel
	myseg = int( myinput[4] / ( myinput[3] * myinput[12] ) ) * myinput[9] * 2 
	i = 4 # box number 
	mybox = myboxpts[i]
	edge = geompy.GetEdge(fuelcell, mybox[0], mybox[3])		# length-y from box
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	if myinput[10]:		# mesh gradient in channels
		myseg = myinput[9]
		mygrad1 = float( myinput[3] /  ( myinput[9] * 2.0 ) )
		mygrad = mygrad1 / myinput[11]
		myfac = myinput[9]/ myinput[13]
				
		# left, bottom
		i = 0
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(myfac * mygrad, myfac * mygrad1)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad, mygrad1)
		algo_local.Propagation()
		# right, bottom
		i = 1
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(myfac * mygrad1, myfac * mygrad)
		algo_local.Propagation()
		# right, top
		i = 2
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel  
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad1, mygrad)
		algo_local.Propagation()
		
	
	else :			# no mesh gradient in channels
		mysegw = myinput[9] # width
		mysegh = myinput[13] # height
		
		
		# first part of channel (left, bottom)
		i = 0
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(mysegw)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(mysegh)
		algo_local.Propagation()
		# second part of channel (right, top)
		i = 2
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(mysegw)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(mysegh)
		algo_local.Propagation()

	## compute mesh
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()
	
	## copy groups from geometry
	for a in allgroups:
		hexa.GroupOnGeom(a)
	
	print "   grid spacing = %f m" % float( myinput[3] /  ( myinput[9] * 2.0 ) )
		
#############################################################################################
### FUNCTION to make MESH	-> without inlet / outlet region
#############################################################################################
def CreateMesh2(myboxes, myboxpts, myinput, fuelcell, allgroups):
	#: Create the mesh
	#: 1) we initialize everything with 1 segment
	#: 2) define number of segments for length of channel
	#: 3) define number of segments for width of channel
	#: 4) define number of segments for height of channel, interconnect, GDL, electrolyte
	global smesh
	
	### first we initialize everything with 1 segment
	hexa = smesh.Mesh(fuelcell)
	hexa_1d = hexa.Segment()
	hexa_1d.NumberOfSegments(1) 
	
	### define number of segments for length of channel as multiplier of the size of segments per channel width  
		
	# channel
	myseg = int( myinput[4] / ( myinput[3] * myinput[12] ) ) * myinput[9] * 2 
	i = 0 # box number 
	mybox = myboxpts[i]
	edge = geompy.GetEdge(fuelcell, mybox[0], mybox[3])		# length-y from box
	algo_local = hexa.Segment(edge)
	algo_local.NumberOfSegments(myseg)
	algo_local.Propagation()
	
	if myinput[10]:		# mesh gradient in channels
		myseg = myinput[9]
		mygrad1 = float( myinput[3] /  ( myinput[9] * 2.0 ) )
		mygrad = mygrad1 / myinput[11]
				
		# left, bottom
		i = 0
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad, mygrad1)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad, mygrad1)
		algo_local.Propagation()
		# right, bottom
		i = 1
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad1, mygrad)
		algo_local.Propagation()
		# right, top
		i = 2
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel  
		algo_local = hexa.Segment(myedge)
		algo_local.StartEndLength(mygrad1, mygrad)
		algo_local.Propagation()
		
	
	else :			# no mesh gradient in channels
		myseg = myinput[9]
		
		# first part of channel (left, bottom)
		i = 0
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(myseg)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(myseg)
		algo_local.Propagation()
		# second part of channel (right, top)
		i = 2
		mybox = myboxpts[i]
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[1])		# width-x of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(myseg)
		algo_local.Propagation()
		myedge = geompy.GetEdge(fuelcell, mybox[0], mybox[6])		# height-z of half channel 
		algo_local = hexa.Segment(myedge)
		algo_local.NumberOfSegments(myseg)
		algo_local.Propagation()

	## compute mesh
	hexa.Quadrangle()
	hexa.Hexahedron()
	hexa.Compute()
	
	## copy groups from geometry
	for a in allgroups:
		hexa.GroupOnGeom(a)
	
	print "   grid spacing = %f m" % float( myinput[3] /  ( myinput[9] * 2.0 ) )

#############################################################################################
#############################################################################################
### END of FUNCTIONS
### here is the main section (as in C-language), where the functions are called in the 
### correct order
#############################################################################################

if myinput[6]:		# we have additional inlet and outlet regions
	CreateCell(myboxes, myboxpts, myinput)		# make geometry (arrange all boxes)
	mychannel = geompy.MakePartition(myboxes)	# add all boxes to a single partition (for view and for mesh)
	geompy.addToStudy(mychannel, 'channel')

	CreateGroups(myboxpts, myinput, mychannel, fluid, myfluidnames, myfacegroups, myfacenames, allgroups)	# Create GROUPS to be addressed later as boundary for OpenFOAM
	CreateMesh(myboxes, myboxpts, myinput, mychannel, allgroups)	# create the mesh and copy the groups from geometry
else:
	CreateCell2(myboxes, myboxpts, myinput)		# make geometry (arrange all boxes)
	mychannel = geompy.MakePartition(myboxes)	# add all boxes to a single partition (for view and for mesh)
	geompy.addToStudy(mychannel, 'channel')

	CreateGroups2(myboxpts, myinput, mychannel, fluid, myfluidnames, myfacegroups, myfacenames, allgroups)	# Create GROUPS to be addressed later as boundary for OpenFOAM
	CreateMesh2(myboxes, myboxpts, myinput, mychannel, allgroups)	# create the mesh and copy the groups from geometry

salome.sg.updateObjBrowser(1)

print "   ... done." 